@@ -8,10 +8,13 @@ use File::Basename qw(basename);
 use File::Path qw(mkpath);
 use HTML::Entities qw(decode_entities);
 use LWP::UserAgent;
+use List::Util qw(max sum);
+use Time::Local ();
 use XML::LibXML;
 use XML::LibXML::XPathContext;
 
 our $default_ctformat = "html";
+our $default_cturl = "http://www.cpantesters.org/show";
 our $Signal = 0;
 
 =encoding utf-8
@@ -22,7 +25,7 @@ CPAN::Testers::ParseReport - parse reports to www.cpantesters.org from various s
 
 =cut
 
-use version; our $VERSION = qv('0.0.12');
+use version; our $VERSION = qv('0.0.13');
 
 =head1 SYNOPSIS
 
@@ -90,6 +93,7 @@ $dumpvar is a hashreference that gets filled with data.
 sub _download_overview {
     my($cts_dir, $distro, %Opt) = @_;
     my $format = $Opt{ctformat} ||= $default_ctformat;
+    my $cturl = $Opt{cturl} ||= $default_cturl;
     my $ctarget = "$cts_dir/$distro.$format";
     my $cheaders = "$cts_dir/$distro.headers";
     if ($Opt{local}) {
@@ -104,7 +108,7 @@ sub _download_overview {
                 print STDERR "(timestamp $timestamp GMT)\n" unless $Opt{quiet};
             }
             print STDERR "Fetching $ctarget..." if $Opt{verbose} && !$Opt{quiet};
-            my $uri = "http://www.cpantesters.org/show/$distro.$format";
+            my $uri = "$cturl/$distro.$format";
             my $resp = _ua->mirror($uri,$ctarget);
             if ($resp->is_success) {
                 print STDERR "DONE\n" if $Opt{verbose} && !$Opt{quiet};
@@ -168,6 +172,7 @@ sub _parse_html {
     if ($Opt{vdistro}) {
         $excuse_string = "selected distro '$Opt{vdistro}'";
         my($fallbacktoversion) = $Opt{vdistro} =~ /(\d+\..*)/;
+        $fallbacktoversion = 0 unless defined $fallbacktoversion;
       RELEASE: for my $i (0..$#releasedivs) {
             my $picked = "";
             my($x) = $nsu ?
@@ -197,7 +202,6 @@ sub _parse_html {
     unless (defined $releasediv) {
         $releasediv = 0;
     }
-    $DB::single=1;
     # using a[1] because a[2] is missing on the first entry
     ($selected_release_distrov) = $nsu ?
         $xc->findvalue("x:h2/x:a[1]/\@name",$releasedivs[$releasediv]) :
@@ -302,6 +306,10 @@ sub parse_distro {
     $Opt{cachedir} ||= "$ENV{HOME}/var/cpantesters";
     my $cts_dir = "$Opt{cachedir}/cpantesters-show";
     mkpath $cts_dir;
+    if ($Opt{solve}) {
+        require Statistics::Regression;
+        $Opt{dumpvars} = "." unless defined $Opt{dumpvars};
+    }
     my $ctarget = _download_overview($cts_dir, $distro, %Opt);
     my $reports;
     $Opt{ctformat} ||= $default_ctformat;
@@ -322,6 +330,9 @@ sub parse_distro {
         print $fh YAML::Syck::Dump(\%dumpvars);
         close $fh or die "Could not close '$dumpfile': $!"
     }
+    if ($Opt{solve}) {
+        solve(\%dumpvars,%Opt);
+    }
 }
 
 =head2 parse_report($target,$dumpvars,%Opt)
@@ -392,7 +403,13 @@ sub parse_report {
     }
     seek $fh, 0, 0;
   LINE: while (<$fh>) {
-        chomp; # reliable line endings?
+        s/\r?\n\z//;
+        while (/!$/) {
+            my $followupline = <$fh>;
+            $followupline =~ s/^\s+//; # remo leading space
+            $_ .= $followupline;
+            s/\r?\n\z//;
+        }
         $_ = decode_entities $_;
         if (/^--------/ && $previous_line[-2] && $previous_line[-2] =~ /^--------/) {
             $current_headline = $previous_line[-1];
@@ -460,8 +477,8 @@ sub parse_report {
                     $extract{"meta:writer"} = $1;
                 } elsif (/CPANPLUS, version (\S+)/) {
                     $extract{"meta:writer"} = "CPANPLUS $1";
-                } elsif (/This report was machine-generated by CPAN::YACSmoke (\S+)/) {
-                    $extract{"meta:writer"} = "CPAN::YACSmoke $1";
+                } elsif (/This report was machine-generated by (\S+) (\S+)/) {
+                    $extract{"meta:writer"} = "$1 $2";
                 }
                 $extract{"meta:writer"} =~ s/[\.,]$// if $extract{"meta:writer"};
             }
@@ -480,10 +497,10 @@ sub parse_report {
             if (/^\s*$/ || m|</pre>|) {
                 $in_summary = 0;
             } else {
-                s/&quot;/"/g;
                 my(%kv) = /\G,?\s*([^=]+)=('[^']+?'|\S+)/gc;
                 while (my($k,$v) = each %kv) {
                     my $ck = "conf:$k";
+                    $ck =~ s/\s+$//;
                     $v =~ s/,$//;
                     if ($v =~ /^'(.*)'$/) {
                         $v = $1;
@@ -492,8 +509,8 @@ sub parse_report {
                     $v =~ s/\s+$//;
                     if ($qr && $ck =~ $qr) {
                         $dumpvars->{$ck}{$v}{$ok}++;
-                    }
-                    if ($conf_vars{$ck}) {
+                        $extract{$ck} = $v;
+                    } elsif ($conf_vars{$ck}) {
                         $extract{$ck} = $v;
                     }
                 }
@@ -596,6 +613,12 @@ sub parse_report {
             $expecting_toolchain_soon=1;
         }
     }                           # LINE
+    if ($Opt{solve}) {
+        $extract{id} = $id;
+        my $data = $dumpvars->{"==DATA=="} ||= [];
+        push @$data, \%extract;
+    }
+    # ---- %extract finished ----
     my $diag = "";
     if (my $qr = $Opt{dumpvars}) {
         $qr = qr/$qr/;
@@ -641,6 +664,128 @@ sub parse_report {
     }
 }
 
+=head2 solve
+
+Feeds a couple of potentially interesting data to
+Statistics::Regression and sorts the result by R^2 descending. Do not
+confuse this with a prove, rather take it as a useful hint. It can
+save you minutes of staring at data and provide a quick overview where
+one should look closer. Displays the N top candidates, where N
+defaults to 3 and can be set with the C<$Opt{solvetop}> variable.
+
+=cut
+{
+    my %never_solve_on = map {($_ => 1)}
+        (
+         "conf:ccflags",
+         "conf:config_args",
+         "conf:cppflags",
+         "conf:lddlflags",
+         "conf:uname",
+         "env:PATH",
+         "env:PERL5LIB",
+         "env:PERL5OPT",
+         'env:$^X',
+         'env:$EGID',
+         'env:$GID',
+         'env:$UID/$EUID',
+         'env:PERL5_CPANPLUS_IS_RUNNING',
+         'env:PERL5_CPAN_IS_RUNNING',
+         'env:PERL5_CPAN_IS_RUNNING_IN_RECURSION',
+        );
+    my %normalize =
+        (
+         id => sub { return shift },
+         'meta:date' => sub {
+             my($Y,$M,$D,$h,$m,$s) = shift =~ /(\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+)/;
+             Time::Local::timegm($s,$m,$h,$D,$M-1,$Y);
+         },
+        );
+sub solve {
+    my($V,%Opt) = @_;
+    require Statistics::Regression;
+    my @regression;
+  VAR: for my $variable (sort keys %$V) {
+        next if $variable eq "==DATA==";
+        if ($never_solve_on{$variable}){
+            warn "Skipping '$variable'\n" unless $Opt{quiet};
+            next VAR;
+        }
+        my $value_distribution = $V->{$variable};
+        my $keys = keys %$value_distribution;
+        my @X = qw(const);
+        if ($normalize{$variable}) {
+            push @X, "n_$variable";
+        } else {
+            for my $value (sort keys %$value_distribution) {
+                my $pf = $value_distribution->{$value};
+                $pf->{PASS} ||= 0;
+                $pf->{FAIL} ||= 0;
+                push @X, "eq_$value";
+                if (
+                    $pf->{PASS} xor $pf->{FAIL}
+                   ) {
+                    my $vl = 40;
+                    substr($value,$vl) = "..." if length $value > 3+$vl;
+                    my $poor_mans_freehand_estimation = 0;
+                    if ($poor_mans_freehand_estimation) {
+                        warn sprintf
+                            (
+                             "%4d %4d %-23s | %s\n",
+                             $pf->{PASS},
+                             $pf->{FAIL},
+                             $variable,
+                             $value,
+                            );
+                    }
+                }
+            }
+        }
+        warn "variable[$variable]keys[$keys]X[@X]\n" unless $Opt{quiet};
+        my $reg = Statistics::Regression->new($variable,\@X);
+      RECORD: for my $rec (@{$V->{"==DATA=="}}) {
+            my $y;
+            if ($rec->{"meta:ok"} eq "PASS") {
+                $y = 1;
+            } elsif ($rec->{"meta:ok"} eq "FAIL") {
+                $y = 0;
+            } else {
+                next RECORD;
+            }
+            my %obs;
+            @obs{@X} = (0) x @X;
+            $obs{const} = 1;
+            for my $x (@X) {
+                if ($x =~ /^eq_(.+)/) {
+                    my $v = $1;
+                    if (exists $rec->{$variable}
+                        && defined $rec->{$variable}
+                        && $rec->{$variable} eq $v) {
+                        $obs{$x} = 1;
+                    }
+                } elsif ($x =~ /^n_(.+)/) {
+                    my $v = $1;
+                    $obs{$x} = $normalize{$v}->($rec->{$v});
+                }
+            }
+            $reg->include($y, \%obs);
+        }
+        eval {$reg->standarderrors};
+        if ($@) {
+            # require YAML::Syck; print STDERR "Line " . __LINE__ . ", File: " . __FILE__ . "\n" . YAML::Syck::Dump({error=>"could not determine standarderrors",variable=>$variable,k=>$reg->k, n=>$reg->n}); # XXX
+        } else {
+            # $reg->print;
+            push @regression, $reg;
+        }
+    }
+    my $top = $Opt{solvetop} || 3;
+    for my $reg (sort {$b->rsq <=> $a->rsq} @regression) {
+        $reg->print;
+        last if --$top <= 0;
+    }
+}
+}
+
 =head1 AUTHOR
 
 Andreas König
